<?php
/**
 * Adds a new special page, Special:Namespaces, to view and modify namespace
 * data.
 * Based on Special:Interwiki (http://www.mediawiki.org/wiki/Extension:SpecialInterwiki)
 * code.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @file
 * @ingroup Extensions
 * @version 2.1
 * @author Stephanie Amanda Stevens <phroziac@gmail.com>
 * @author SPQRobin <robin_1273@hotmail.com>
 * @author Stephen Kennedy
 * @copyright Copyright © 2005-2007 Stephanie Amanda Stevens
 * @copyright Copyright © 2006 Stephen Kennedy
 * @copyright Copyright © 2007 SPQRobin
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License 2.0 or later
 */

if ( !defined( 'MEDIAWIKI' ) ) {
	die( 'This is not a valid entry point to MediaWiki.' );
}

// Extension credits that will show up on Special:Version
$wgExtensionCredits['specialpage'][] = array(
	'path' => __FILE__,
	'name' => 'SpecialNamespaces',
	'version' => '2.1',
	'author' => array( 'Stephanie Amanda Stevens', 'SPQRobin', 'others' ),
	'url' => 'http://mediawiki.org/wiki/Extension:SpecialNamespaces',
	'description' => 'Adds a [[Special:Namespaces|special page]] to view and edit the namespaces table.',
	'descriptionmsg' => 'namespaces-desc',
);

// Set up the new special page
$dir = dirname( __FILE__ ) . '/';
$wgExtensionMessagesFiles['Namespaces'] = $dir . 'SpecialNamespaces.i18n.php';
$wgExtensionAliasesFiles['Namespaces'] = $dir . 'SpecialNamespaces.alias.php';

$wgAutoloadClasses['SpecialNamespaces'] = $dir . 'SpecialNamespaces_body.php';
$wgSpecialPages['Namespaces'] = 'SpecialNamespaces';
$wgSpecialPageGroups['Namespaces'] = 'wiki';

// New user right, required to modify namespaces
$wgAvailableRights[] = 'namespaces';

// Set up logging to log all changes to namespace data
$wgLogTypes[] = 'namespaces';
$wgLogNames['namespaces'] = 'namespaces_logpagename';
$wgLogHeaders['namespaces'] = 'namespaces_logpagetext';
$wgLogActions['namespaces/namespaces'] = 'namespaces_logentry';
$wgLogActions['namespaces/ns_add'] = 'namespaces_log_added';
$wgLogActions['namespaces/ns_delete'] = 'namespaces_log_deleted';
$wgLogActions['namespaces/ns_edit'] = 'namespaces_log_edited';

// hook to retrieve $wgExtraNamespaces[...] from cached database table (namespace_names)
$wgHooks['LanguageGetMagic'][] = 'fnNamespaceHook';

/**
 * Code for hook function to retrieve $wgExtraNamespaces[...] from database.
 * Use memcached if available to avoid severe performance penalty for multiple
 * DB accesses.
 */
function fnNamespaceHook( &$magicWords, $langCode ) {
	global $wgExtraNamespaces, $wgNamespaceAliases;
	global $wgDBname, $wgMemc;
	global $wgSitename, $wgMetaNamespace, $wgMetaNamespaceTalk;

	/*
	if ( $wgExtraNamespaces != null ) {
		return true;
	}
	*/

	if ( $wgExtraNamespaces == null ) {
		$wgExtraNamespaces = array();
	}

	if ( $wgNamespaceAliases == null ) {
		$wgNamespaceAliases = array();
	}

	$key = "$wgDBname:SpecialNamespaces:name";
	$cached = $wgMemc->get( $key );

	if ( !is_array( $cached ) ) {
		$dbr = wfGetDB( DB_SLAVE );
		$res = $dbr->select( 'namespace_names', '*', array(), __METHOD__ );
		$numrows = $dbr->numRows( $res );
		if ( $numrows > 0 ) // @todo FIXME: braces per [[Manual:Coding conventions]]!
			foreach ( $res as $s ) {
				$nsindex = htmlspecialchars( $s->ns_id );
				$nsname = htmlspecialchars( $s->ns_name );
				$nscanonical = htmlspecialchars( $s->ns_canonical );
				$nsdefault = htmlspecialchars( $s->ns_default );
				$nsname = str_replace( ' ', '_', $nsname );
				$nsname = str_replace( ':', '', $nsname );

				if ( $nsdefault > 0 ) {
					$wgExtraNamespaces[$nsindex] = $nsname;
				} else {
					$wgNamespaceAliases[$nsname] = $nsindex;
				}
				if ( $nscanonical > 0) {
					if ( $nsindex == 4 ) {
						$wgSitename = $nsname;
						$wgMetaNamespace = str_replace( ' ', '_', $nsname );
					}
					if ( $nsindex == 5 ) {
						$wgMetaNamespaceTalk = str_replace( ' ', '_', $nsname );
					}
				}
			}
			$cached = array();
			$cached['ns'] = $wgExtraNamespaces;
			$cached['aka'] = $wgNamespaceAliases;
			$cached['site'] = $wgSitename;
			$cached['project'] = $wgMetaNamespace;
			$cached['prjtalk'] = $wgMetaNamespaceTalk;
			$wgMemc->set( $key, $cached );
	} else {
		$wgExtraNamespaces = $cached['ns'];
		$wgNamespaceAliases = $cached['aka'];
		$wgSitename = $cached['site'];
		$wgMetaNamespace = $cached['project'];
		$wgMetaNamespaceTalk = $cached['prjtalk'];
	}
	return true;
 }
?>
