<?php
/**
 * Implements Special:Namespaces
 * @ingroup SpecialPage
 */
class SpecialNamespaces extends SpecialPage {

	/**
	 * Constructor -- set up the new special page
	 */
	public function __construct() {
		parent::__construct( 'Namespaces' );
	}

	/**
	 * Delete memcached key.
	 */
	function discard() {
		global $wgDBname, $wgMemc;
		$wgMemc->delete( "$wgDBname:SpecialNamespaces:name" );
	}

	/**
	 * Show the special page
	 *
	 * @param $par Mixed: parameter passed to the special page or null
	 */
	public function execute( $par ) {
		global $wgRequest, $wgOut, $wgUser;

		$this->setHeaders();
		$this->outputHeader();

		$admin = $wgUser->isAllowed( 'namespaces' );
		if ( $admin ) {
			$wgOut->setPageTitle( wfMsg( 'namespaces' ) );
		} else {
			$wgOut->setPageTitle( wfMsg( 'namespaces-title-norights' ) );
		}
		$action = $wgRequest->getVal( 'action', $par );

		switch( $action ) {
			case 'delete':
			case 'edit':
			case 'add':
				// Check permissions
				if( !$admin ) {
					$wgOut->permissionRequired('namespaces');
					return;
				}
				// Is the database in read-only mode?
				if( wfReadOnly() ) {
					$wgOut->readOnlyPage();
					return;
				}
				$this->showForm( $action );
				break;
			case 'submit':
				// Check permissions
				if( !$admin ) {
					$wgOut->permissionRequired( 'namespaces' );
					return;
				}
				// Is the database in read-only mode?
				if( wfReadOnly() ) {
					$wgOut->readOnlyPage();
					return;
				}
				// Prevent cross-site request forgeries
				if( !$wgRequest->wasPosted() || !$wgUser->matchEditToken( $wgRequest->getVal( 'wpEditToken' ) ) ) {
					$wgOut->addWikiMsg( 'sessionfailure' );
					return;
				}
				$this->doSubmit();
				break;
			default:
				$this->showList( $admin );
				break;
		}
	}

	function showForm( $action ) {
		global $wgRequest, $wgUser, $wgOut;

		$actionUrl = $this->getTitle()->getLocalURL( 'action=submit' );
		$token = $wgUser->editToken();
		$defaultreason = $wgRequest->getVal( 'wpNamespacesReason',
			wfMsgForContent( 'namespaces_defaultreason' )
		);

		switch( $action ) {
			case 'delete':
				$nsid = $wgRequest->getVal( 'prefix' );
				$button = wfMsg( 'delete' );
				$topmessage = wfMsg( 'namespaces_delquestion', $nsid );
				$deletingmessage = wfMsgExt( 'namespaces_deleting', array( 'parseinline' ), $nsid );
				$reasonmessage = wfMsg( 'deletecomment' );

				$wgOut->addHTML(
					Xml::openElement( 'fieldset' ) .
					Xml::element( 'legend', null, $topmessage ) .
					Xml::openElement( 'form', array('id'=> 'mw-namespaces-deleteform', 'method'=> 'post', 'action' => $actionUrl ) ) .
					Xml::openElement( 'table' ) .
					"<tr><td>$deletingmessage</td></tr>".
					'<tr><td class="mw-label">' . Xml::label( $reasonmessage, 'mw-namespaces-deletereason') . '</td>' .
					'<td class="mw-input">' .
					Xml::input( 'wpNamespacesReason', 60, $defaultreason, array( 'tabindex' => '1', 'id' => 'mw-namespaces-deletereason', 'maxlength' => '200' ) ) .
					'</td></tr>' .
					'<tr><td class="mw-submit">' . Xml::submitButton( $button, array( 'id' => 'mw-namespaces-submit' ) ) .
					Xml::hidden( 'wpNamespacesID', $nsid ) .
					Xml::hidden( 'wpNamespacesAction', $action ) .
					Xml::hidden( 'wpEditToken', $token ) .
					'</td></tr>' .
					Xml::closeElement( 'table' ) .
					Xml::closeElement( 'form' ) .
					Xml::closeElement( 'fieldset' )
				);
				break;
			case 'edit':
			case 'add':
				if( $action == 'edit' ) {
					$nsid = $wgRequest->getVal( 'prefix' );
					$dbr = wfGetDB( DB_SLAVE );
					$row = $dbr->selectRow(
						'namespace_names',
						'*',
						array( 'ns_id' => $nsid ),
						__METHOD__
					);
					if( !$row ) {
						$this->error( 'namespaces_editerror', $nsid );
						return;
					}
					$nsid = '<tt>' . htmlspecialchars( $row->ns_id ) . '</tt>';
					$defaultname = $row->ns_name;
					$nsdefault = $row->ns_default;
					$nscanonical = $row->ns_canonical;
					$old = Xml::hidden( 'wpNamespacesID', $row->ns_id );
					$topmessage = wfMsgExt( 'namespaces_edittext', array( 'parseinline' ) );
					$intromessage = wfMsgExt( 'namespaces_editintro', array( 'parseinline' ) );
					$button = wfMsg( 'edit' );
				} else {
					$nsid = $wgRequest->getVal( 'wpNamespacesID' ) ? $wgRequest->getVal( 'wpNamespacesID' ) : $wgRequest->getVal( 'prefix' );
					$nsid = Xml::input( 'wpNamespacesID', 20, $nsid, array( 'tabindex' => '1', 'id' => 'mw-namespaces-nsid', 'maxlength' => '20' ) );
					$nsdefault = $wgRequest->getCheck( 'wpNamespacesDefault' );
					$nscanonical = $wgRequest->getCheck( 'wpNamespacesCanonical' );
					$old = '';
					$defaultname = $wgRequest->getVal( 'wpNamespacesName' ) ? $wgRequest->getVal( 'wpNamespacesName' ) : wfMsg( 'namespaces_defaultname' );
					$topmessage = wfMsgExt( 'namespaces_addtext', array( 'parseinline' ) );
					$intromessage = wfMsgExt( 'namespaces_addintro', array( 'parseinline' ) );
					$button = wfMsg( 'namespaces_addbutton' );
				}

				$nsidmessage = wfMsgHtml( 'namespaces_nsid' );
				$nsdefaultmessage = wfMsg( 'namespaces_default' );
				$nscanonicalmessage = wfMsg( 'namespaces_canonical' );
				$reasonmessage = wfMsg( 'namespaces_reasonfield' );
				$nsnamemessage = wfMsg( 'namespaces_nsname' );

				$wgOut->addHTML(
					Xml::openElement( 'fieldset' ) .
					Xml::element( 'legend', null, $topmessage ) .
					$intromessage .
					Xml::openElement( 'form', array( 'id' => 'mw-namespaces-editform', 'method' => 'post', 'action' => $actionUrl ) ) .
					Xml::openElement( 'table', array( 'id' => "mw-namespaces-$action" ) ) .
					"<tr><td class=\"mw-label\">$nsidmessage</td><td><tt>$nsid</tt></td></tr>" .
					'<tr><td class="mw-label">' . Xml::label( $nsdefaultmessage, 'mw-namespaces-nsdefault' ) . '</td>' .
					"<td class='mw-input'>" . Xml::check( 'wpNamespacesDefault', $nsdefault, array( 'id' => 'mw-namespaces-nsdefault' ) ) . '</td></tr>' .
					'<tr><td class="mw-label">' . Xml::label( $nscanonicalmessage, 'mw-namespaces-nscanonical' ) . '</td>' .
					'<td class="mw-input">' .  Xml::check( 'wpNamespacesCanonical', $nscanonical, array( 'id' => 'mw-namespaces-nscanonical' ) ) . '</td></tr>' .
					'<tr><td class="mw-label">' . Xml::label( $nsnamemessage, 'mw-namespaces-nsname' ) . '</td>' .
					'<td class="mw-input">' . Xml::input( 'wpNamespacesName', 60, $defaultname, array( 'tabindex' => '1', 'maxlength' => '200', 'id' => 'mw-namespaces-nsname' ) ) . '</td></tr>' .
					'<tr><td class="mw-label">' . Xml::label( $reasonmessage, 'mw-namespaces-editreason' ) . '</td>' .
					'<td class="mw-input">' . Xml::input( 'wpNamespacesReason', 60, $defaultreason, array( 'tabindex' => '1', 'id' => 'mw-namespaces-editreason', 'maxlength' => '200') ) .
					Xml::hidden( 'wpNamespacesAction', $action ) .
					$old .
					Xml::hidden( 'wpEditToken', $token ) .
					'</td></tr>' .
					'<tr><td class="mw-submit">' . Xml::submitButton( $button, array( 'id' => 'mw-namespaces-submit' ) ) . '</td></tr>' .
					Xml::closeElement( 'table' ) .
					Xml::closeElement( 'form' ) .
					Xml::closeElement( 'fieldset' )
				);
				break;
		}
	}

	function doSubmit() {
		global $wgRequest, $wgOut;
		$nsid = $wgRequest->getVal( 'wpNamespacesID' );
		$do = $wgRequest->getVal( 'wpNamespacesAction' );
		if( preg_match( '/[\s:&=]/', $nsid ) ) {
			$this->error( 'namespaces-badprefix', htmlspecialchars( $nsid ) );
			$this->showForm( $do );
			return;
		}
		$reason = $wgRequest->getText( 'wpNamespacesReason' );
		$selfTitle = $this->getTitle();
		$dbw = wfGetDB( DB_MASTER );
		switch( $do ) {
			case 'delete':
				$dbw->delete(
					'namespace_names',
					array( 'ns_id' => $nsid ),
					__METHOD__
				);

				if ( $dbw->affectedRows() == 0 ) {
					$this->error( 'namespaces_delfailed', $nsid );
					$this->showForm( $do );
				} else {
					$wgOut->addWikiMsg( 'namespaces_deleted', $nsid );
					$wgOut->returnToMain( false, $selfTitle );
					$log = new LogPage( 'namespaces' );
					$log->addEntry( 'ns_delete', $selfTitle, $reason, array( $nsid ) );
				}
				$this->discard();
				break;
			case 'edit':
			case 'add':
				$theurl = $wgRequest->getVal( 'wpNamespacesName' );
				$nsdefault = $wgRequest->getCheck( 'wpNamespacesDefault' ) ? 1 : 0;
				$nscanonical = $wgRequest->getCheck( 'wpNamespacesCanonical' ) ? 1 : 0;
				$data = array(
					'ns_id' => $nsid,
					'ns_name' => $theurl,
					'ns_default' => $nsdefault,
					'ns_canonical' => $nscanonical
				);

				if( $do == 'add' ) {
					$dbw->insert(
						'namespace_names',
						$data,
						__METHOD__,
						'IGNORE'
					);
				} else {
					$dbw->update(
						'namespace_names',
						$data,
						array( 'ns_id' => $nsid ),
						__METHOD__,
						'IGNORE'
					);
				}

				if( $dbw->affectedRows() == 0 ) {
					$this->error( "namespaces_{$do}failed", $nsid );
					$this->showForm( $do );
				} else {
					$wgOut->addWikiMsg( "namespaces_{$do}ed", $nsid );
					$wgOut->returnToMain( false, $selfTitle );
					$log = new LogPage( 'namespaces' );
					$log->addEntry(
						'ns_' . $do, $selfTitle, $reason,
						array( $nsid, $theurl, $nsdefault, $nscanonical )
					);
				}
				$this->discard();
				break;
		}
	}

	function trans_default( $tl, $msg0, $msg1 ) {
		if( $tl === '0' ) {
			return $msg0;
		}
		if( $tl === '1' ) {
			return $msg1;
		}
		return htmlspecialchars( $tl );
	}

	function showList( $admin ) {
		global $wgUser, $wgOut, $wgScriptPath;

		$wgOut->addExtensionStyle( "{$wgScriptPath}/extensions/Namespaces/SpecialNamespaces.css" );

		$nsidmessage = wfMsgHtml( 'namespaces_nsid' );
		$nsnamemessage = wfMsgHtml( 'namespaces_nsname' );
		$nsdefaultmessage = wfMsgHtml( 'namespaces_default' );
		$nscanonicalmessage = wfMsgHtml( 'namespaces_canonical' );
		$message_0 = wfMsgHtml( 'namespaces_0' );
		$message_1 = wfMsgHtml( 'namespaces_1' );

		$out = '
<table width="100%" cellspacing="0" cellpadding="0" border="0" style="border:0" class="mw-namespacestable wikitable intro">
<tr><th class="mw-align-left">' . $nsidmessage . '</th><td>' . wfMsgExt( 'namespaces_nsid_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-left">' . $nsnamemessage . '</th><td>' . wfMsgExt( 'namespaces_nsname_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-left">' . $nsdefaultmessage . '</th><td>' . wfMsgExt( 'namespaces_default_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-right">' . $message_1 . '</th><td>' . wfMsgExt( 'namespaces_default_1_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-right">' . $message_0 . '</th><td>' . wfMsgExt( 'namespaces_default_0_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-left">' . $nscanonicalmessage . '</th><td>' . wfMsgExt( 'namespaces_canonical_intro', 'parseinline' ) . '</td></tr>';
		if ( 0 ) { // yay unused code!
			$out .= '
<tr><th class="mw-align-right">' . $message_1 . '</th><td>' . wfMsgExt( 'namespaces_canonical_1_intro', 'parseinline' ) . '</td></tr>
<tr><th class="mw-align-right">' . $message_0 . '</th><td>' . wfMsgExt( 'namespaces_canonical_0_intro', 'parseinline' ) . '</td></tr>
';
		}
		$out .= '</table>';
		$wgOut->addWikiMsg( 'namespaces_intro' );
		$wgOut->addHTML( $out );
		$wgOut->addWikiMsg( 'namespaces_intro_footer' );
		$selfTitle = $this->getTitle();

		if ( $admin ) {
			$skin = $wgUser->getSkin();
			$addtext = wfMsgHtml( 'namespaces_addtext' );
			$addlink = $skin->link( $selfTitle, $addtext, array(), array( 'action' => 'add' ) );
			$wgOut->addHTML( '<p>' . $addlink . '</p>' );
		}

		$dbr = wfGetDB( DB_SLAVE );
		$res = $dbr->select(
			'namespace_names',
			'*',
			1,
			__METHOD__,
			array( 'ORDER BY' => 'ns_id' )
		);
		$numrows = $res->numRows();
		if ( $numrows == 0 ) {
			$this->error( 'namespaces_error' );
			return;
		}

		$out = "
			<table width='100%' class='mw-namespacestable wikitable body'>
			<tr id='namespacestable-header'><th>$nsidmessage</th> <th>$nsnamemessage</th> <th>$nsdefaultmessage</th> <th>$nscanonicalmessage</th>";
		if( $admin ) {
			$deletemessage = wfMsgHtml( 'delete' );
			$editmessage = wfMsgHtml( 'edit' );
			$out .= '<th>' . wfMsgHtml( 'namespaces_edit' ) . '</th>';
		}
		$out .= "</tr>\n";

		while( $s = $res->fetchObject() ) {
			$nsid = htmlspecialchars( $s->ns_id );
			$nsname = htmlspecialchars( $s->ns_name );
			$nsdefault = $this->trans_default( $s->ns_default, $message_0, $message_1 );
			$nscanonical = $this->trans_default( $s->ns_canonical, $message_0, $message_1 );
			$out .= "<tr class='mw-namespacestable-row'>
					<td class='mw-namespacestable-nsid'>$nsid</td>
					<td class='mw-namespacestable-nsname'>$nsname</td>
					<td class='mw-namespacestable-nsdefault'>$nsdefault</td>
					<td class='mw-namespacestable-nscanonical'>$nscanonical</td>";
			if( $admin ) {
				$out .= '<td class="mw-namespacestable-modify">';
				$out .= $skin->link( $selfTitle, $editmessage, array(),
						array( 'action' => 'edit', 'prefix' => $s->ns_id ) );
				$out .= ', ';
				$out .= $skin->link( $selfTitle, $deletemessage, array(),
						array( 'action' => 'delete', 'prefix' => $s->ns_id ) );
				$out .= '</td>';
			}

			$out .= "\n</tr>\n";
		}
		$res->free();
		$out .= '</table><br />';
		$wgOut->addHTML( $out );
	}

	function error() {
		global $wgOut;
		$args = func_get_args();
		$wgOut->wrapWikiMsg( "<p class=\"error\">$1</p>", $args );
	}
}
?>
